@@ -52,10 +52,10 @@
 
 extern char **environ;
 
-#define GDM_SESSION_WORKER_JOB_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_SESSION_WORKER_JOB, GdmSessionWorkerJobPrivate))
-
-struct GdmSessionWorkerJobPrivate
+struct _GdmSessionWorkerJob
 {
+        GObject         parent;
+
         char           *command;
         GPid            pid;
         gboolean        for_reauth;
@@ -82,10 +82,6 @@ enum {
 
 static guint signals [LAST_SIGNAL] = { 0, };
 
-static void     gdm_session_worker_job_class_init       (GdmSessionWorkerJobClass *klass);
-static void     gdm_session_worker_job_init     (GdmSessionWorkerJob      *session_worker_job);
-static void     gdm_session_worker_job_finalize (GObject         *object);
-
 G_DEFINE_TYPE (GdmSessionWorkerJob, gdm_session_worker_job, G_TYPE_OBJECT)
 
 static void
@@ -117,12 +113,19 @@ session_worker_job_setup_journal_fds (void)
 static void
 session_worker_job_child_setup (GdmSessionWorkerJob *session_worker_job)
 {
+        sigset_t mask;
         session_worker_job_setup_journal_fds ();
 
         /* Terminate the process when the parent dies */
 #ifdef HAVE_SYS_PRCTL_H
         prctl (PR_SET_PDEATHSIG, SIGTERM);
 #endif
+        /*
+         * Reset signal mask to default since it was altered by the
+         * manager process
+         */
+        sigemptyset (&mask);
+        sigprocmask (SIG_SETMASK, &mask, NULL);
 }
 
 static void
@@ -139,8 +142,8 @@ session_worker_job_child_watch (GPid                 pid,
                  : WIFSIGNALED (status) ? WTERMSIG (status)
                  : -1);
 
-        g_spawn_close_pid (job->priv->pid);
-        job->priv->pid = -1;
+        g_spawn_close_pid (job->pid);
+        job->pid = -1;
 
         if (WIFEXITED (status)) {
                 int code = WEXITSTATUS (status);
@@ -169,99 +172,89 @@ static void
 copy_environment_to_hash (GdmSessionWorkerJob *job,
                           GHashTable          *hash)
 {
-        char **environment;
+        g_auto(GStrv) environment = NULL;
         gint   i;
 
-        if (job->priv->environment != NULL) {
-                environment = g_strdupv (job->priv->environment);
+        if (job->environment != NULL) {
+                environment = g_strdupv (job->environment);
         } else {
                 environment = g_get_environ ();
         }
+
         for (i = 0; environment[i]; i++) {
-                char **parts;
+                g_auto(GStrv) parts = NULL;
 
                 parts = g_strsplit (environment[i], "=", 2);
 
                 if (parts[0] != NULL && parts[1] != NULL) {
                         g_hash_table_insert (hash, g_strdup (parts[0]), g_strdup (parts[1]));
                 }
-
-                g_strfreev (parts);
         }
-
-        g_strfreev (environment);
 }
 
 static GPtrArray *
 get_job_arguments (GdmSessionWorkerJob *job,
                    const char          *name)
 {
-        GPtrArray  *args;
-        GError     *error;
-        char      **argv;
+        g_autoptr(GPtrArray) args = NULL;
+        g_autoptr(GError) error = NULL;
+        g_auto(GStrv) argv = NULL;
         int         i;
 
         args = NULL;
-        argv = NULL;
-        error = NULL;
-        if (! g_shell_parse_argv (job->priv->command, NULL, &argv, &error)) {
+        if (!g_shell_parse_argv (job->command, NULL, &argv, &error)) {
                 g_warning ("Could not parse command: %s", error->message);
-                g_error_free (error);
-                goto out;
+                return NULL;
         }
 
-        args = g_ptr_array_new ();
+        args = g_ptr_array_new_with_free_func (g_free);
         g_ptr_array_add (args, g_strdup (argv[0]));
         g_ptr_array_add (args, g_strdup (name));
         for (i = 1; argv[i] != NULL; i++) {
                 g_ptr_array_add (args, g_strdup (argv[i]));
         }
-        g_strfreev (argv);
 
         g_ptr_array_add (args, NULL);
-out:
-        return args;
+
+        return g_steal_pointer (&args);
 }
 
 static GPtrArray *
 get_job_environment (GdmSessionWorkerJob *job)
 {
-        GPtrArray     *env;
-        GHashTable    *hash;
+        g_autoptr(GPtrArray) env = NULL;
+        g_autoptr(GHashTable) hash = NULL;
 
-        env = g_ptr_array_new ();
+        env = g_ptr_array_new_with_free_func (g_free);
 
         /* create a hash table of current environment, then update keys has necessary */
         hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
         copy_environment_to_hash (job, hash);
 
-        g_hash_table_insert (hash, g_strdup ("GDM_SESSION_DBUS_ADDRESS"), g_strdup (job->priv->server_address));
+        g_hash_table_insert (hash, g_strdup ("GDM_SESSION_DBUS_ADDRESS"), g_strdup (job->server_address));
 
-        if (job->priv->for_reauth) {
+        if (job->for_reauth) {
                 g_hash_table_insert (hash, g_strdup ("GDM_SESSION_FOR_REAUTH"), g_strdup ("1"));
         }
 
         g_hash_table_foreach (hash, (GHFunc)listify_hash, env);
-        g_hash_table_destroy (hash);
 
         g_ptr_array_add (env, NULL);
 
-        return env;
+        return g_steal_pointer (&env);
 }
 
 static gboolean
 gdm_session_worker_job_spawn (GdmSessionWorkerJob *session_worker_job,
                               const char          *name)
 {
-        GError          *error;
+        g_autoptr(GError) error = NULL;
+        g_autoptr(GPtrArray) args = NULL;
+        g_autoptr(GPtrArray) env = NULL;
         gboolean         ret;
-        GPtrArray       *args;
-        GPtrArray       *env;
-
-        ret = FALSE;
 
         g_debug ("GdmSessionWorkerJob: Running session_worker_job process: %s %s",
-                 name != NULL? name : "", session_worker_job->priv->command);
+                 name != NULL? name : "", session_worker_job->command);
 
         args = get_job_arguments (session_worker_job, name);
 
@@ -270,37 +263,29 @@ gdm_session_worker_job_spawn (GdmSessionWorkerJob *session_worker_job,
         }
         env = get_job_environment (session_worker_job);
 
-        error = NULL;
         ret = g_spawn_async_with_pipes (NULL,
                                         (char **) args->pdata,
                                         (char **)env->pdata,
                                         G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_FILE_AND_ARGV_ZERO,
                                         (GSpawnChildSetupFunc)session_worker_job_child_setup,
                                         session_worker_job,
-                                        &session_worker_job->priv->pid,
+                                        &session_worker_job->pid,
                                         NULL,
                                         NULL,
                                         NULL,
                                         &error);
 
-        g_ptr_array_foreach (args, (GFunc)g_free, NULL);
-        g_ptr_array_free (args, TRUE);
-
-        g_ptr_array_foreach (env, (GFunc)g_free, NULL);
-        g_ptr_array_free (env, TRUE);
-
         if (! ret) {
                 g_warning ("Could not start command '%s': %s",
-                           session_worker_job->priv->command,
+                           session_worker_job->command,
                            error->message);
-                g_error_free (error);
         } else {
-                g_debug ("GdmSessionWorkerJob: : SessionWorkerJob on pid %d", (int)session_worker_job->priv->pid);
+                g_debug ("GdmSessionWorkerJob: : SessionWorkerJob on pid %d", (int)session_worker_job->pid);
         }
 
-        session_worker_job->priv->child_watch_id = g_child_watch_add (session_worker_job->priv->pid,
-                                                                      (GChildWatchFunc)session_worker_job_child_watch,
-                                                                      session_worker_job);
+        session_worker_job->child_watch_id = g_child_watch_add (session_worker_job->pid,
+                                                                (GChildWatchFunc)session_worker_job_child_watch,
+                                                                session_worker_job);
 
         return ret;
 }
@@ -315,7 +300,9 @@ gboolean
 gdm_session_worker_job_start (GdmSessionWorkerJob *session_worker_job,
                               const char          *name)
 {
-        gboolean    res;
+        gboolean res;
+
+        g_return_val_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job), FALSE);
 
         g_debug ("GdmSessionWorkerJob: Starting worker...");
 
@@ -329,8 +316,8 @@ handle_session_worker_job_death (GdmSessionWorkerJob *session_worker_job)
 {
         int exit_status;
 
-        g_debug ("GdmSessionWorkerJob: Waiting on process %d", session_worker_job->priv->pid);
-        exit_status = gdm_wait_on_and_disown_pid (session_worker_job->priv->pid, 5);
+        g_debug ("GdmSessionWorkerJob: Waiting on process %d", session_worker_job->pid);
+        exit_status = gdm_wait_on_and_disown_pid (session_worker_job->pid, 5);
 
         if (WIFEXITED (exit_status) && (WEXITSTATUS (exit_status) != 0)) {
                 g_debug ("GdmSessionWorkerJob: Wait on child process failed");
@@ -338,8 +325,8 @@ handle_session_worker_job_death (GdmSessionWorkerJob *session_worker_job)
                 /* exited normally */
         }
 
-        g_spawn_close_pid (session_worker_job->priv->pid);
-        session_worker_job->priv->pid = -1;
+        g_spawn_close_pid (session_worker_job->pid);
+        session_worker_job->pid = -1;
 
         g_debug ("GdmSessionWorkerJob: SessionWorkerJob died");
 }
@@ -347,15 +334,14 @@ handle_session_worker_job_death (GdmSessionWorkerJob *session_worker_job)
 void
 gdm_session_worker_job_stop_now (GdmSessionWorkerJob *session_worker_job)
 {
-        if (session_worker_job->priv->pid <= 1) {
+        g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
+
+        if (session_worker_job->pid <= 1) {
                 return;
         }
 
         /* remove watch source before we can wait on child */
-        if (session_worker_job->priv->child_watch_id > 0) {
-                g_source_remove (session_worker_job->priv->child_watch_id);
-                session_worker_job->priv->child_watch_id = 0;
-        }
+        g_clear_handle_id (&session_worker_job->child_watch_id, g_source_remove);
 
         gdm_session_worker_job_stop (session_worker_job);
         handle_session_worker_job_death (session_worker_job);
@@ -366,13 +352,15 @@ gdm_session_worker_job_stop (GdmSessionWorkerJob *session_worker_job)
 {
         int res;
 
-        if (session_worker_job->priv->pid <= 1) {
+        g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
+
+        if (session_worker_job->pid <= 1) {
                 return;
         }
 
-        g_debug ("GdmSessionWorkerJob: Stopping job pid:%d", session_worker_job->priv->pid);
+        g_debug ("GdmSessionWorkerJob: Stopping job pid:%d", session_worker_job->pid);
 
-        res = gdm_signal_pid (session_worker_job->priv->pid, SIGTERM);
+        res = gdm_signal_pid (session_worker_job->pid, SIGTERM);
 
         if (res < 0) {
                 g_warning ("Unable to kill session worker process");
@@ -383,7 +371,7 @@ GPid
 gdm_session_worker_job_get_pid (GdmSessionWorkerJob *session_worker_job)
 {
         g_return_val_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job), 0);
-        return session_worker_job->priv->pid;
+        return session_worker_job->pid;
 }
 
 void
@@ -392,8 +380,8 @@ gdm_session_worker_job_set_server_address (GdmSessionWorkerJob *session_worker_j
 {
         g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
 
-        g_free (session_worker_job->priv->server_address);
-        session_worker_job->priv->server_address = g_strdup (address);
+        g_free (session_worker_job->server_address);
+        session_worker_job->server_address = g_strdup (address);
 }
 
 void
@@ -402,7 +390,7 @@ gdm_session_worker_job_set_for_reauth (GdmSessionWorkerJob *session_worker_job,
 {
         g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
 
-        session_worker_job->priv->for_reauth = for_reauth;
+        session_worker_job->for_reauth = for_reauth;
 }
 
 void
@@ -411,7 +399,25 @@ gdm_session_worker_job_set_environment (GdmSessionWorkerJob *session_worker_job,
 {
         g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
 
-        session_worker_job->priv->environment = g_strdupv ((char **) environment);
+        session_worker_job->environment = g_strdupv ((char **) environment);
+}
+
+static void
+gdm_session_worker_job_finalize (GObject *object)
+{
+        GdmSessionWorkerJob *session_worker_job;
+
+        g_return_if_fail (object != NULL);
+        g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (object));
+
+        session_worker_job = GDM_SESSION_WORKER_JOB (object);
+
+        gdm_session_worker_job_stop (session_worker_job);
+
+        g_free (session_worker_job->command);
+        g_free (session_worker_job->server_address);
+
+        G_OBJECT_CLASS (gdm_session_worker_job_parent_class)->finalize (object);
 }
 
 static void
@@ -452,13 +458,13 @@ gdm_session_worker_job_get_property (GObject    *object,
 
         switch (prop_id) {
         case PROP_SERVER_ADDRESS:
-                g_value_set_string (value, self->priv->server_address);
+                g_value_set_string (value, self->server_address);
                 break;
         case PROP_FOR_REAUTH:
-                g_value_set_boolean (value, self->priv->for_reauth);
+                g_value_set_boolean (value, self->for_reauth);
                 break;
         case PROP_ENVIRONMENT:
-                g_value_set_pointer (value, self->priv->environment);
+                g_value_set_pointer (value, self->environment);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -466,18 +472,12 @@ gdm_session_worker_job_get_property (GObject    *object,
         }
 }
 
-static GObject *
-gdm_session_worker_job_constructor (GType                  type,
-                                    guint                  n_construct_properties,
-                                    GObjectConstructParam *construct_properties)
+static void
+gdm_session_worker_job_init (GdmSessionWorkerJob *session_worker_job)
 {
-        GdmSessionWorkerJob      *session_worker_job;
+        session_worker_job->pid = -1;
 
-        session_worker_job = GDM_SESSION_WORKER_JOB (G_OBJECT_CLASS (gdm_session_worker_job_parent_class)->constructor (type,
-                                                                                       n_construct_properties,
-                                                                                       construct_properties));
-
-        return G_OBJECT (session_worker_job);
+        session_worker_job->command = g_strdup (LIBEXECDIR "/gdm-session-worker");
 }
 
 static void
@@ -487,36 +487,33 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
 
         object_class->get_property = gdm_session_worker_job_get_property;
         object_class->set_property = gdm_session_worker_job_set_property;
-        object_class->constructor = gdm_session_worker_job_constructor;
         object_class->finalize = gdm_session_worker_job_finalize;
 
-        g_type_class_add_private (klass, sizeof (GdmSessionWorkerJobPrivate));
-
         g_object_class_install_property (object_class,
                                          PROP_SERVER_ADDRESS,
                                          g_param_spec_string ("server-address",
                                                               "server address",
                                                               "server address",
                                                               NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
         g_object_class_install_property (object_class,
                                          PROP_FOR_REAUTH,
                                          g_param_spec_boolean ("for-reauth",
                                                                "for reauth",
                                                                "for reauth",
                                                                FALSE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
         g_object_class_install_property (object_class,
                                          PROP_ENVIRONMENT,
                                          g_param_spec_pointer ("environment",
                                                                "environment",
                                                                "environment",
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
         signals [STARTED] =
                 g_signal_new ("started",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (GdmSessionWorkerJobClass, started),
+                              0,
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__VOID,
@@ -526,7 +523,7 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
                 g_signal_new ("exited",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (GdmSessionWorkerJobClass, exited),
+                              0,
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__INT,
@@ -538,7 +535,7 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
                 g_signal_new ("died",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (GdmSessionWorkerJobClass, died),
+                              0,
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__INT,
@@ -547,37 +544,6 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
                               G_TYPE_INT);
 }
 
-static void
-gdm_session_worker_job_init (GdmSessionWorkerJob *session_worker_job)
-{
-
-        session_worker_job->priv = GDM_SESSION_WORKER_JOB_GET_PRIVATE (session_worker_job);
-
-        session_worker_job->priv->pid = -1;
-
-        session_worker_job->priv->command = g_strdup (LIBEXECDIR "/gdm-session-worker");
-}
-
-static void
-gdm_session_worker_job_finalize (GObject *object)
-{
-        GdmSessionWorkerJob *session_worker_job;
-
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (object));
-
-        session_worker_job = GDM_SESSION_WORKER_JOB (object);
-
-        g_return_if_fail (session_worker_job->priv != NULL);
-
-        gdm_session_worker_job_stop (session_worker_job);
-
-        g_free (session_worker_job->priv->command);
-        g_free (session_worker_job->priv->server_address);
-
-        G_OBJECT_CLASS (gdm_session_worker_job_parent_class)->finalize (object);
-}
-
 GdmSessionWorkerJob *
 gdm_session_worker_job_new (void)
 {
